<html>

<head>
    <title>reCAPTCHA demo: Simple page</title>
    <script src="https://www.google.com/recaptcha/api.js" async defer></script>
</head>

<body>
    <form action="{% url 'captcha_view' %}" method="POST">
        {% csrf_token %}
        <div class="g-recaptcha" data-sitekey="{{public_key}}"></div>
        <br />
        <input type="submit" value="Submit">
    </form>
</body>

</html>